__version__ = "0.0.13"

"""jedi_toolz is a python package containing many utilities to simplify working
with data.

jedi_toolz is divided into the following sub-modules:

config: Provides helper functions to extract the contents of a .ini
configuration file.

data: Provides many helper functions for checking the type of data passed and
converting dict objects to tables and record objects.

domo: Connects to a DOMO instance using credentials defined in a .ini config
file and provides several helper functions.

show: Provides several functions for printing tabular data. Utilized the
tabulate package.

xlsx: Provides functions for exporting and formatting data in a .xlsx file. Uses
openpyxl to export and read data.

str_funcs: Provides utility functions for format strings primarily used as column
names into valid or prettier formats by adding or removing underscores,
spaces, etc.
"""

from jedi_toolz import config
from jedi_toolz.show import *
from jedi_toolz import xlsx
from jedi_toolz import domo
from jedi_toolz.data import *
from jedi_toolz.str_funcs import *


# AUTOGENERATED! DO NOT EDIT! File to edit: domo.ipynb (unless otherwise specified).

__all__ = ['connect', 'tables', 'get_id', 'query']

# Internal Cell
import pydomo
from pydomo.datasets import Sorting
from jedi_toolz import config
import io
import csv

__tables__ = None

def connect() -> pydomo.Domo:
    """Returns a connection object to a DOMO instance."""
    client_id = config.select("domo", "client_id")
    secret = config.select("domo", "secret")
    return pydomo.Domo(client_id, secret)

def tables():
    """Returns a dataframe of all datasets for which the
    user has share permissions or is marked as the owner.
    """
    global __tables__
    if __tables__ is not None: return __tables__

    domo = connect()
    datasets = domo.datasets.list(sort=Sorting.NAME)
    result = []
    for table in datasets:
        table["owner_id"] = table["owner"]["id"]
        table["owner"] = table["owner"]["name"]
        result.append(table)
    __tables__ = result
    return __tables__

def get_id(table_name) -> str:
    """Returns the 'id' for a given table_name."""
    for table in tables():
        if table["name"] == table_name:
            return table["id"]

def query(table_name):
    """Takes a 'table_name' and returns a pandas.DataFrame
    object containing the rows and columns of a domo
    dataset.
    """
    id = get_id(table_name)
    domo = connect()
    include_csv_headers = True
    result = domo.datasets.data_export(
        id, include_csv_headers)
    file = io.StringIO(result)
    rows = csv.DictReader(file)
    return [row for row in rows]


# AUTOGENERATED! DO NOT EDIT! File to edit: config.ipynb (unless otherwise specified).

__all__ = ['set_path', 'as_dict', 'as_records', 'select', 'example_ini']

# Internal Cell
from configparser import ConfigParser
import os
from pathlib import Path
from typing import Dict, List, Optional, Union

environ = "JEDI_TOOLZ_PATH"

PathOrStr = Optional[Union[str,Path]]

def set_path(config_path: PathOrStr=None) -> Path:
    """Returns the path to the jedi_toolz.ini file or the path
    to the config_path.ini file.
    """
    if config_path:
        return Path(config_path).expanduser()
    elif os.getenv(environ):
        return Path(os.getenv(environ)).expanduser()
    else:
        raise ValueError(
            f"Either {environ} environment variable must exist "
            "or config_path must be provided."
        )


def as_dict(config_path: PathOrStr=None) -> Dict[str,str]:
    """Returns configurations as a nested dict. Keys and values are strings."""
    location = set_path(config_path)
    cfg = ConfigParser()
    cfg.read(location)
    sections = cfg.sections()
    result = {}
    for section in sections:
        options = {k: v for k, v in cfg[section].items()}
        result[section] = options
    return result

def as_records(config_path: PathOrStr=None) -> List[Dict[str,str]]:
    """Returns configurations as a list of dicts."""
    return [
        {"section": section, "option": option, "value": value}
        for section, options in as_dict(config_path).items()
        for option, value in options.items()
    ]

def select(section: str, option: Optional[str]=None, config_path: PathOrStr=None) -> Union[Dict[str, str], str]:
    """Returns the configuration a nested dict or value."""
    settings = as_dict(config_path)
    if option:
        return settings[section][option]
    else:
        return settings[section]

def example_ini():
    """Creates an example.ini file and returns it's path."""
    path = Path.cwd() / "tests/example.ini"
    with path.open(mode="w") as file:
        file.writelines([
            "[test1]\n",
            "value1 = 5\n",
            "value2 = 6\n",
            "\n",
            "[test2]\n",
            "valuea = ABC\n",
            "valueb = 123\n",
        ])
    return path